#include "snes-to-usb.h"
#include "ctrlr.h"

typedef enum {
	CTRLR_STATE_IDLE,		// no ongoing read
	CTRLR_STATE_LOADH,		// load is high
	CTRLR_STATE_LOADL,		// load is low
	CTRLR_STATE_CLKH,		// clock is high
	CTRLR_STATE_CLKL		// clock is low
} ctrlr_state_t;

ctrlr_state_t ctrlr_state = CTRLR_STATE_IDLE;

// starting value for timer, for counting bit periods
#define TIMER_VAL 256-8

// keeps track of current bit being processed
uint16_t bit_pos;


void ctrlr_init (void) {
	// set clock and load pins as outputs
	CTRLR_PORT_DIR |= CTRLR_CLK | CTRLR_LOAD;
	
	// set up timer to feed from system clock,
	// with no prescaling
	TCCR0B |= B(CS00);
	TCCR0B &= ~B(CS01);
	TCCR0B &= ~B(CS02);
}

void ctrlr_start_read (void) {
	// set load pin high
	CTRLR_PORT |= CTRLR_LOAD;
	// start timing
	TIFR0 |= TOV0;
	TCNT0 = TIMER_VAL;
	ctrlr_state = CTRLR_STATE_LOADH;
}

bool ctrlr_read_poll (uint16_t *buttons) {
	// if the timer period has expired...
	if (TIFR0 & B(TOV0)) {
		// then clear the overflow flag
		TIFR0 |= TOV0;
		// and run the state machine
		switch (ctrlr_state) {
			case CTRLR_STATE_LOADH:
				// when the load-high-time period has expired,
				// set load low and start the timer again
				CTRLR_PORT &= ~CTRLR_LOAD;
				TCNT0 = TIMER_VAL;
				ctrlr_state = CTRLR_STATE_LOADL;
				break;
		
			case CTRLR_STATE_LOADL:
				// when the low period for load is up,
				// first store the value of the first bit
				if (CTRLR_PORT_IN & CTRLR_DATA) *buttons &= ~0x8000;
				else *buttons |= 0x8000;
				// init the bit position indicator with the position of the next bit
				bit_pos = 0x4000;
				// now set the clock line high, and start the timer again
				CTRLR_PORT |= CTRLR_CLK;
				TCNT0 = TIMER_VAL;
				ctrlr_state = CTRLR_STATE_CLKH;
				break;

			case CTRLR_STATE_CLKH:
				// set the clock line low
				CTRLR_PORT &= ~CTRLR_CLK;
				// store the current bit
				if (CTRLR_PORT_IN & CTRLR_DATA) *buttons &= ~bit_pos;
				else *buttons |= bit_pos;
				// restart timer
				TCNT0 = TIMER_VAL;
				ctrlr_state = CTRLR_STATE_CLKL;
				break;
		
			case CTRLR_STATE_CLKL:
				// look at the bit position we just processed
				if (bit_pos == 0x0001) {
					// if that's the entire word, then we're done
					ctrlr_state = CTRLR_STATE_IDLE;
					return true;
				}
				// if there's more to go, set the clock line high again
				CTRLR_PORT |= CTRLR_CLK;
				// move to the next bit position
				bit_pos = bit_pos >> 1;
				// and restart the timer
				TCNT0 = TIMER_VAL;
				ctrlr_state = CTRLR_STATE_CLKH;
				break;

			default:
				return false;
		}
	}
	
	return false;	
}

